-- for debugging purposes
local function d(...)
	local t = {...}
	for k, v in next, t do
		t[k] = tostring(v)
	end
	DEFAULT_CHAT_FRAME:AddMessage("quip debug: " .. table.concat(t, ", "))
end

local pairs = pairs

local itemCache
local scanners = {}

local function setToString(setName)
	return Quip.getColor(setName) .. setName .. "|r"
end

-- API
Quip = {}
function Quip.invalidate()
	itemCache = nil
end

function Quip.setScanner(addon, f)
	scanners[addon] = f
end

function Quip.addBinding(addon, setName, itemId)
	itemCache = itemCache or {}

	local s = itemCache[itemId]
	local s2 = setToString(setName)
	if s then
		s = s .. ", " .. s2
	else
		s = s2
	end
	itemCache[itemId] = s
end

function Quip.getItemIdFromItemLink(itemLink)
	if type(itemLink) == "string" then
		local m = itemLink:match("^(%d+):")
		if m then
			return m + 0
		end
	end
end

-- end of API

local function getSetsForItem(itemId)
	if itemId then
		if itemCache == nil then
			itemCache = {}
			for addon, f in pairs(scanners) do
				f()
			end
		end
		return itemCache[itemId]
	end
end

local title = string.format(ITEM_SET_BONUS, "")

local TipHooker = LibStub("LibTipHooker-1.1")
local function processTooltip(self, name, link)
	if link then
		local itemId = 0 + link:match("item:(%d+):")
		local sets = getSetsForItem(itemId)
		if sets then
			self:AddDoubleLine(title, sets, 0.4, 0.6, 0.8, 1, 1, 1)
			self:Show()
		end
	end
end
TipHooker:Hook(processTooltip, "item")

--[=[
local function onTooltipSetItem(self)
	local _, hyperlink = self:GetItem()
	if hyperlink then
		processTooltip(self, nil, hyperlink)
	end
end


local frame = CreateFrame("Frame")
frame:RegisterEvent("ADDON_LOADED")
frame:SetScript("OnEvent", function(self, event, arg1) 
	if arg1 ~= "Quip" then
		return
	end
	self:UnregisterEvent("ADDON_LOADED")
	local hookedTooltips = {}

	local mt_index = getmetatable(GameTooltip).__index
	local function hook(method)
		local old = mt_index[method]
		if old then
			mt_index[method] = function(self, ...)
				if not hookedTooltips[self] then
					hookedTooltips[self] = true
					self:HookScript("OnTooltipSetItem", onTooltipSetItem)
				end
				self[method] = old
				return old(self, ...)
			end
		end
	end

	local hooks = [[
	SetAuctionItem
	SetAuctionSellItem
	SetBagItem
	SetBuybackItem
	SetHyperlink
	SetHyperlinkCompareItem
	SetInboxItem
	SetInventoryItem
	SetLootItem
	SetLootRollItem
	SetMerchantItem
	SetQuestItem
	SetQuestLogItem
	SetQuestLogSpecialItem
	SetSendMailItem
	SetSocketedItem
	SetTradePlayerItem
	SetTradeSkillItem
	SetTradeTargetItem
	]]
	for s in hooks:gmatch("[A-Za-z]+") do
		hook(s)
	end
end)
--]=]

