local function component(c, q, p)
	if c < 1/6 then
		return p + ((q - p) * 6 * c)
	end
	if c < 1/2 then
		return q
	end
	if c < 2/3 then
		return p + ((q - p) * 6 * (2/3 - c))
	end
	return p
end

-- all ranges 0 - 1
local function hslToRgb(h, s, l)
	local q = (l < 1/2) and l * (1 + s) or l + s - l * s
	local p = 2 * l - q
	local tr = (h + 1 + 1/3) % 1
	local tg = (h + 1) % 1
	local tb = (h + 1 - 1/3) % 1
	
	-- red
	return component(tr, q, p), component(tg, q, p), component(tb, q, p)
end

local byte = string.byte
local function hashString(s)
	local v = 0
	local n = #s
	for i = 1, n do
		v = 31 * v + byte(s, i)
	end
	return (v % 360) / 360
end

local colorCache = {}

local function setColor(s, r, g, b)
	color = string.format("|cff%02x%02x%02x", 255*r, 255*g, 255*b)
	colorCache[s] = color
	return color
end

local function calcColor(s)
	local v = hashString(s)
	local r, g, b = hslToRgb(v, 0.73, 0.74)
	return r, g, b
end


function Quip.getColor(s)
	local color = colorCache[s]
	if not color then
		local r, g, b = calcColor(s)
		color = setColor(s, r, g, b)
	end
	return color
end

local function checkRange(c)
	assert(c >= 0 and c<= 1, "Color out of range")
end

function Quip.setColor(s, r, g, b)
	checkRange(r)
	checkRange(g)
	checkRange(b)
	if not (r and g and b) then
		r, g, b = calcColor(s)
	end
	setColor(s, r, g, b)
	Quip.invalidate()
end

